#'Shift longitudes in maps
#'
#'@description This function shifts the longitudes in maps.
#'
#'@param data An array with, at least, 'lon_dim' dimension.
#'@param lon A vector with the longitudes.
#'@param westB A number with the west boundary for the new longitudes.
#'@param lon_dim A string with the name of the longitude dimension.
#'  The default value is "lon".
#'@param ncores The number of cores used for computation. The default 
#'  value is NULL (it uses only one core).
#'
#'@return 
#'\item{data}{
#'  Array with the shifted data with the same dimensions as parameter 'data'.
#'}
#'\item{lon}{
#'  The new longitudes with the same length as parameter 'lon'.
#'}
#'
#'@import multiApply
#'@examples
#'data <- array(data = 1:50, dim = c(lon = 360, lat = 181))
#'lon <- array(data = 0:359, dim = c(lon = 360))
#'lat <- -90:90 ## lat does not change
#'shifted <- ShiftLon(data = data, lon = lon, westB = -180, ncores = 1)
#'s2dv::PlotEquiMap(var = data, lon = lon, lat = lat, filled.continents = FALSE)
#'s2dv::PlotEquiMap(var = shifted$data, lon = shifted$lon, lat = lat, filled.continents = FALSE)
#'
#'@import multiApply
#'@export
ShiftLon <- function(data, lon,  westB, lon_dim = 'lon', ncores = NULL) {
  
  ## Check inputs
  ## data
  if (!is.array(data)){
    stop("Parameter 'data' must be and array.") 
  }  
  ## lon
  if (!is.numeric(lon)){
    stop("Parameter 'lon' must be numeric.") 
  } else if (!(length(lon) == as.numeric(dim(data)[lon_dim]))){
    stop("The length of 'lon' must coindide with the length of 'lon_dim' in 'data'.")
  }
  ## westB
  if (!is.numeric(westB)){
    stop("Parameter 'westB' must be numeric.") 
  }
  ## lon_dim
  if (!is.character(lon_dim)){
    stop("Parameter 'lon_dim' must be a character string.")
  }
  if (!(lon_dim %in% names(dim(data)))){
    stop("Parameter 'data' must have 'lon_dim' dimension.") 
  }
  ## ncores
  if (!is.null(ncores)) {
    if (!is.numeric(ncores) | any(ncores %% 1 != 0) | any(ncores < 0) |
        length(ncores) > 1) {
      stop("Parameter 'ncores' must be a positive integer.")
    }
  }
  
  #########--- Shifting the longitudes (only needed to do it once) ---############
  ## adjust western boundary to lon vector if necessary
  if (min(abs(westB-lon))>90){
    westB <- ifelse(westB>0,westB-360,westB+360)
  }
  ## find closest index in lon vector
  first <- which.min(abs(westB-lon))[1]
  if (first==1){
    new.lon <- lon
  } else {
    new.lon <- c(lon[first:length(lon)],lon[1:(first-1)])
  }
  ## order to  monotonically increasing
  if (!all(diff(new.lon)>0)){
    new.lon[1:(which(diff(new.lon)<0))] <- new.lon[1:(which(diff(new.lon)<0))]-360
    if (all(new.lon<0)){
      new.lon <- new.lon+360
    }
  }
  
  ############--- Shifting the data ---############
  output <- multiApply::Apply(data = data, 
                              target_dims = lon_dim,
                              fun = .ShiftLon, 
                              lon = lon, 
                              new.lon = new.lon, 
                              ncores = ncores)$output1
  return(list(data = output, 
              lon = new.lon))
}

.ShiftLon <- function(data, lon, new.lon) {
  
  new.data <- NA * data
  new.data[new.lon %in% lon] <- data[lon %in% new.lon, drop = F]
  new.data[!new.lon %in% lon] <- data[!lon %in% new.lon, drop = F]
  
  return(new.data)
}
